""" About information for videodb sdk"""


__version__ = "0.2.6"
__title__ = "videodb"
__author__ = "videodb"
__email__ = "contact@videodb.io"
__url__ = "https://github.com/video-db/videodb-python"
__license__ = "Apache License 2.0"
